﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using UnityEngine;

namespace Mod
{
    public class BloodySyringe : SyringeBehaviour
    {

        public static void Load()
        {
            ModAPI.RegisterLiquid(BloodySyringeSerum.ID, new BloodySyringeSerum());

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),

                    NameOverride = "​<b></b>" + "Bloody Syringe",
                    DescriptionOverride = "Syringe that removes all liquids that are not blood and their effects.",
                    ThumbnailOverride = ModAPI.LoadSprite("sprites/invc.png"),

                    CategoryOverride = ModAPI.FindCategory("Polars Bear"),
                    AfterSpawn = (Instance) => { Destroy(Instance.GetComponent<SyringeBehaviour>()); Instance.GetOrAddComponent<BloodySyringe>(); }
                }
            );
        }
        public override string GetLiquidID() => BloodySyringeSerum.ID;
        public class BloodySyringeSerum : Liquid
        {
            public const string ID = "​<b></b>" + "FLUID RESTORING AGENT";
            public BloodySyringeSerum()
            {
                Color = new UnityEngine.Color(1.00000f, 1.00000f, 1.0000f);
            }
            public override void OnEnterLimb(LimbBehaviour limb) 
            {
                limb.gameObject.GetOrAddComponent<BloodFixingBehaviour>();
            }
            public override void OnExitContainer(BloodContainer container) { }
            public override void OnUpdate(BloodContainer container) { }
            public override void OnEnterContainer(BloodContainer container) { }

            private class BloodFixingBehaviour : MonoBehaviour
            {
                LimbBehaviour limb;
                CirculationBehaviour circ;
                Dictionary<Color, KeyValuePair<GameObject, BloodParticleFakingBehaviour>> particles = new Dictionary<Color, KeyValuePair<GameObject, BloodParticleFakingBehaviour>>();
                Color color;
                float counter = 0f;

                void Awake()
                {
                    limb = GetComponent<LimbBehaviour>();
                    circ = limb.CirculationBehaviour;
                    limb.CirculationBehaviour.StopAllCoroutines();
                    bool hadOutsider = false;
                    color = Color.black;
                    foreach (KeyValuePair<Liquid, RefFloat> juice in circ.LiquidDistribution)
                    {
                        if (juice.Key == Liquid.GetLiquid(ID) || juice.Key == limb.GetOriginalBloodType()) { continue; }
                        float d = juice.Value.Raw / (circ.TotalLiquidAmount - circ.GetAmount(limb.GetOriginalBloodType()) - circ.GetAmount(Liquid.GetLiquid(ID)));
                        color += juice.Key.Color * d;
                        hadOutsider = true;
                        Debug.Log(juice.Key);
                    }
                    if (!hadOutsider || circ.TotalLiquidAmount == circ.GetAmount(limb.GetOriginalBloodType()) + circ.GetAmount(Liquid.GetLiquid(ID)))
                    {
                        circ.RemoveLiquid(Liquid.GetLiquid(ID), circ.GetAmount(Liquid.GetLiquid(ID)));
                        Destroy(this);
                    }
                    color.a = 1f;
                    color.r = Mathf.Clamp01(color.r);
                    color.g = Mathf.Clamp01(color.g);
                    color.b = Mathf.Clamp01(color.b);
                    Debug.Log(color);
                    Vector2 worldPosition = transform.position;
                    Vector2 direction = new Vector2(0f, -1f);
                    worldPosition = limb.PhysicalBehaviour.spriteRenderer.bounds.ClosestPoint(worldPosition);
                    GameObject go = Instantiate(limb.Person.BleedingParticlePrefab, worldPosition - direction * 0.1f, Quaternion.identity, transform);
                    go.transform.up = direction;
                    Destroy(go.GetComponent<BleedingParticleBehaviour>());
                    BloodParticleFakingBehaviour component = go.AddComponent<BloodParticleFakingBehaviour>();
                    component.ShouldBecomeSmokeInWater = true;
                    component.CirculationBehaviour = circ;
                    particles.Add(color, new KeyValuePair<GameObject, BloodParticleFakingBehaviour>(go,component));
                }

                void Update()
                {
                    color = Color.black;
                    bool hadOutsider = false;
                    foreach (KeyValuePair<Liquid, RefFloat> juice in circ.LiquidDistribution)
                    {
                        if (juice.Key == Liquid.GetLiquid(ID) || juice.Key == limb.GetOriginalBloodType()) { continue; }
                        float d = juice.Value.Raw / (circ.TotalLiquidAmount - circ.GetAmount(limb.GetOriginalBloodType()) - circ.GetAmount(Liquid.GetLiquid(ID)));
                        color += juice.Key.Color * d;
                        circ.RemoveLiquid(juice.Key, Math.Min(0.1f * Time.deltaTime, juice.Value.Raw));
                        hadOutsider = true;
                    }
                    foreach (KeyValuePair<Color, KeyValuePair<GameObject, BloodParticleFakingBehaviour>> kv in particles)
                    {
                        BloodParticleFakingBehaviour p = kv.Value.Value;
                        p.computedColor = color;
                    }
                    if (!hadOutsider || circ.TotalLiquidAmount == circ.GetAmount(limb.GetOriginalBloodType()) + circ.GetAmount(Liquid.GetLiquid(ID)))
                    {
                        circ.RemoveLiquid(Liquid.GetLiquid(ID), circ.GetAmount(Liquid.GetLiquid(ID)));
                        foreach (KeyValuePair<Color, KeyValuePair<GameObject, BloodParticleFakingBehaviour>> kv in particles)
                        {
                            Destroy(kv.Value.Key);
                            particles.Remove(color);
                        }
                        Destroy(this);
                    }
                }

                private class BloodParticleFakingBehaviour : MonoBehaviour
                {
                    private void Awake()
                    {
                        this.BloodParticleSystem = base.GetComponent<ParticleSystem>();
                        this.main = this.BloodParticleSystem.main;
                        this.emission = this.BloodParticleSystem.emission;
                    }
                    private void Start()
                    {
                        if (!this.ShouldBecomeSmokeInWater && this.WaterPuffSystem)
                        {
                            UnityEngine.Object.Destroy(this.WaterPuffSystem);
                        }
                    }
                    private void Update()
                    {
                        float num = 1f;
                        bool flag = WaterBehaviour.IsPointUnderWater(base.transform.position);
                        this.main.startSpeedMultiplier = this.SpeedMultiplier * UnityEngine.Random.Range(0.14f, 0.2f);
                        this.main.startColor = computedColor;
                        float num2 = (this.CirculationBehaviour.TotalLiquidAmount > float.Epsilon) ? Mathf.Min(num, 1f) : 0f;
                        this.emission.rateOverTimeMultiplier = (flag ? 0f : (24f * num2 * this.RateMultiplier));
                        this.emission.enabled = true;
                        if (this.BloodSplatController)
                        {
                            this.BloodSplatController.DecalColourMultiplier = computedColor;
                            this.BloodSplatController.EjectionForce = new Vector2(0.25f, 0.35f) * this.main.startSpeedMultiplier;
                            this.BloodSplatController.SpawningChance = (flag ? 0f : (this.CirculationBehaviour.TotalLiquidAmount * 0.5f));
                        }
                        if (this.ShouldBecomeSmokeInWater && this.WaterPuffSystem)
                        {
                            var ma = this.WaterPuffSystem.main;
                            ma.startColor = computedColor;
                            if (flag)
                            {
                                var em = this.WaterPuffSystem.emission;
                                em.rateOverTimeMultiplier = num2;
                                if (!this.WaterPuffSystem.isPlaying)
                                {
                                    this.WaterPuffSystem.Play();
                                    return;
                                }
                            }
                            else if (this.WaterPuffSystem.isPlaying)
                            {
                                this.WaterPuffSystem.Stop();
                                return;
                            }
                        }
                    }
                    public CirculationBehaviour CirculationBehaviour;
                    public ParticleSystem BloodParticleSystem;
                    public ParticleSystem WaterPuffSystem;
                    public FancyBloodSplatController BloodSplatController;
                    public bool IsOnlyForInternalBleeding = false;
                    private ParticleSystem.MainModule main;
                    private ParticleSystem.EmissionModule emission;
                    public float SpeedMultiplier = 1f;
                    public float RateMultiplier = 1f;
                    public bool ShouldBecomeSmokeInWater = true;
                    public Color computedColor = Color.clear;
                }
            }
        }

    }

}

//Author: Polars Bear
