﻿using System;
using UnityEngine;
using Object = System.Object;
using Obj = UnityEngine.Object;


public class Nailgun
{
	Camera camera;
	Transform center;
	Transform background;
	int framesOnSceneBegin = 0;

	public Nailgun()
	{
		camera = ThumbnailCreator.Main.Camera;
		center = ThumbnailCreator.Main.Center;
		background = ThumbnailCreator.Main.Background;
		framesOnSceneBegin = Time.frameCount;
	}

	class WaitForFirstFrame : MonoBehaviour
	{
		public Func<Object[], bool> func;
		public Object[] args;

		public static GameObject Wait(Func<Object[], bool> function, Object[] arguments, string name)
		{
			GameObject obj = new GameObject("Waiter for " + name);
			WaitForFirstFrame wait = obj.AddComponent<WaitForFirstFrame>();
			wait.func = function;
			wait.args = arguments;
			return obj;
		}

		void Start()
		{
			func.DynamicInvoke(new Object[] { args });
			Destroy(this);
		}
	}

	private static bool ThumbWaited(Object[] args)
	{
		Nailgun nailer = (Nailgun)args[0];
		string spawnableName = (string)args[1];
		GameObject obj = (GameObject)args[2];
		int resolution = (int)args[3];
		bool useBackground = (bool)args[4];
		Texture2D backgroundTexture = (Texture2D)args[5];
		Texture2D defaultTexture = (Texture2D)args[6];

		nailer.Thumb(spawnableName, obj, resolution, useBackground, backgroundTexture, defaultTexture, true);
		return true;
	}

	private Sprite Thumb(string spawnableName, GameObject obj, int resolution, bool useBackground, Texture2D backgroundTexture, Texture2D defaultTexture, bool waited)
	{
		Sprite spr = Sprite.Create(CreateThumbnail(obj, resolution, useBackground, backgroundTexture, defaultTexture, waited), new Rect(0, 0, resolution, resolution), Vector2.one / 2);
		ModAPI.FindSpawnable(spawnableName).ViewSprite = spr;
		return spr;
	}

	private static bool SyringeNailWaited(Object[] args)
	{
		Nailgun nailer = (Nailgun)args[0];
		string spawnableName = (string)args[1];
		Type syringeType = (Type)args[2];
		int resolution = (int)args[3];
		bool useBackground = (bool)args[4];
		Texture2D backgroundTexture = (Texture2D)args[5];
		Texture2D defaultTexture = (Texture2D)args[6];

		nailer.SyringeNail(spawnableName, syringeType, resolution, useBackground, backgroundTexture, defaultTexture, true);
		return true;
	}

	static Func<Object[], bool> syringeNailWaitedFunc = new Func<Object[], bool>(SyringeNailWaited);

	public Sprite SyringeNail(string spawnableName, Type syringeType, int resolution = 256, bool useBackground = false, Texture2D backgroundTexture = null, Texture2D defaultTexture = null, bool waited = false)
	{
		if (Time.frameCount - framesOnSceneBegin <= 0 && !waited)
		{
			WaitForFirstFrame.Wait(syringeNailWaitedFunc, new Object[] { this, spawnableName, syringeType, resolution, useBackground, backgroundTexture, defaultTexture }, spawnableName);
		}
		else
		{
			GameObject obj = Obj.Instantiate(ModAPI.FindSpawnable(spawnableName).Prefab);
			obj.name = spawnableName;
			obj.transform.localScale *= 2.25f;
			Obj.Destroy(obj.GetComponent<SyringeBehaviour>());
			SyringeBehaviour syr = (SyringeBehaviour)obj.AddComponent(syringeType);
			syr.PressureMode = BloodContainer.PressureDirection.None;
			return Nail(spawnableName, obj, resolution, useBackground, backgroundTexture, defaultTexture, waited);
		}
		return null;
	}

	static Func<Object[], bool> thumbWaitedFunc = new Func<Object[], bool>(ThumbWaited);
	Sprite Nail(string spawnableName, GameObject obj, int resolution, bool useBackground = false, Texture2D backgroundTexture = null, Texture2D defaultTexture = null, bool waited = false)
	{
		if (Time.frameCount - framesOnSceneBegin <= 1 && !waited)
		{
			WaitForFirstFrame.Wait(thumbWaitedFunc, new Object[] { this, spawnableName, obj, resolution, useBackground, backgroundTexture, defaultTexture }, spawnableName);
		}
		else
		{
			return Thumb(spawnableName, obj, resolution, useBackground, backgroundTexture, defaultTexture, waited);
		}
		return null;
	}

	Texture2D CreateThumbnail(GameObject obj, int resolution, bool useBackground = false, Texture2D backgroundTexture = null, Texture2D defaultTexture = null, bool waited = false)
	{
		Texture2D result = defaultTexture ? defaultTexture : Texture2D.blackTexture;
		try
		{
			try
			{
				obj.BroadcastMessage("Start", SendMessageOptions.DontRequireReceiver);

				obj.BroadcastMessage("Update", SendMessageOptions.DontRequireReceiver);
				obj.BroadcastMessage("FixedUpdate", SendMessageOptions.DontRequireReceiver);
			}
			catch (Exception e)
            {
				Debug.LogWarning("Nailgun encountered an issue on " + obj.name + ". But don't worry, errors like these are common, and should only be fixed if they cause real issue.\n" + e.Message);
			}

			obj.transform.position = center.position;
			obj.transform.rotation = Quaternion.Euler(0f, 0f, 45f);

			foreach (Transform transform in obj.GetComponentsInChildren<Transform>())
			{
				if (transform.gameObject.layer != 15)
				{
					transform.gameObject.layer = camera.gameObject.layer;
				}
			}
			if (backgroundTexture)
			{
				useBackground = true;
				background.GetComponent<SpriteRenderer>().sprite = Sprite.Create(backgroundTexture, new Rect(0f, 0f, backgroundTexture.width, backgroundTexture.height), Vector2.one / 2);
			}

			background.gameObject.layer = useBackground ? 15 : 16;

			result = TakePicture(obj, resolution);
			background.gameObject.layer = 15;

			Obj.DestroyImmediate(obj);
		}
		catch (Exception)
		{
			throw;
		}
		finally
		{
			camera.gameObject.SetActive(false);
		}
		return result;
	}

	Texture2D TakePicture(GameObject go, int resolution)
	{
		camera.gameObject.SetActive(true);
		Vector2 vector;
		vector.x = go.transform.position.x - ThumbnailPadding;
		vector.y = go.transform.position.y + ThumbnailPadding;
		Vector2 vector2;
		vector2.x = go.transform.position.x + ThumbnailPadding;
		vector2.y = go.transform.position.y - ThumbnailPadding;

		float num = Mathf.Max(Mathf.Abs(vector.x - vector2.x), Mathf.Abs(vector.y - vector2.y));
		camera.orthographicSize = num / 2f;
		background.localScale = Vector3.one * camera.orthographicSize;
		RenderTexture temporary = RenderTexture.GetTemporary(resolution, resolution, 16);
		camera.targetTexture = temporary;
		camera.Render();
		RenderTexture.active = camera.targetTexture;
		Texture2D texture2D = new Texture2D(resolution, resolution, TextureFormat.RGB24, false);
		texture2D.ReadPixels(new Rect(0f, 0f, (float)resolution, (float)resolution), 0, 0);
		texture2D.Apply();
		RenderTexture.active = null;
		camera.targetTexture = null;
		RenderTexture.ReleaseTemporary(temporary);
		camera.gameObject.SetActive(false);
		return texture2D;
	}

	public static float ThumbnailPadding = 1f;
}

// Author: Polars Bear
