﻿using System;
using System.Collections;
using System.Reflection;
using UnityEngine;

namespace Mod
{
    public class CloneSyringe : SyringeBehaviour, Messages.ILodged, Messages.IBreak
    {
        public const string name = "​<b></b>" + "Cloning Syringe";
        public PersonBehaviour cloneTargetPerson = null;
        public GameObject cloneTarget = null;
        Liquid liquid;
        public static void Load()
        {
            ModAPI.RegisterLiquid(CloneSyringeSerum.ID, new CloneSyringeSerum());

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),

                    NameOverride = name,
                    DescriptionOverride = "Copies the user when injected, then pastes them when broken.",

                    CategoryOverride = ModAPI.FindCategory("Polars Bear"),
                    AfterSpawn = (Instance) => { Destroy(Instance.GetComponent<SyringeBehaviour>()); Instance.GetOrAddComponent<CloneSyringe>(); }
                }
            );
            Mod.nailgun.SyringeNail(name, typeof(CloneSyringe));
        }

        void Awake()
        {
            base.Awake();
            liquid = Liquid.GetLiquid(GetLiquidID());
        }
        public override void Break(Vector2 velocity)
        {
            Clone();
            base.Break(velocity);
        }
        private void Clone()
        {
            if (!cloneTarget)
            {
                return;
            }
            foreach (LimbBehaviour limb in cloneTargetPerson.Limbs)
            {
                limb.CirculationBehaviour.RemoveLiquid(liquid, limb.CirculationBehaviour.GetAmount(liquid));
            }
            cloneTarget.transform.position = transform.position;
            cloneTarget.transform.localPosition += Vector3.up * (cloneTarget.transform.localScale.y * 1.5f);
            cloneTarget.SetActive(true);
        }


        public override void OnFragmentHit(float force)
        {
            Clone();
            base.OnFragmentHit(force);
        }
        public override void Lodged(Stabbing stabbing)
        {
            cloneTarget = Instantiate(stabbing.victim.GetComponent<LimbBehaviour>().Person.gameObject);
            cloneTargetPerson = cloneTarget.GetComponent<PersonBehaviour>();
            cloneTarget.SetActive(false);
            base.Lodged(stabbing);
        }

        public override void Shot(Shot shot)
        {
            Clone();
            base.Shot(shot);
        }
        public override void ExitShot(Shot shot)
        {
            Clone();
            base.ExitShot(shot);
        }

        public override string GetLiquidID() => CloneSyringeSerum.ID;
        public class CloneSyringeSerum : Liquid
        {
            public const string ID = "​<b></b>" + "CLONING SERUM";
            public CloneSyringeSerum()
            {
                Color = new Color(0.75f, 0.75f, 1f);
            }
            public override void OnEnterLimb(LimbBehaviour limby)
            {
                foreach (LimbBehaviour limb in limby.Person.Limbs)
                {
                    limb.CirculationBehaviour.RemoveLiquid(this, limb.CirculationBehaviour.GetAmount(this));
                }
            }
            public override void OnExitContainer(BloodContainer container) { }
            public override void OnUpdate(BloodContainer container) { }
            public override void OnEnterContainer(BloodContainer container) { }
        }

    }
}

//Author: Polars Bear
