﻿using System;
using System.Collections;
using System.Reflection;
using UnityEngine;

namespace Mod
{
    public class LukewarmSyringe : SyringeBehaviour
    {
        public const string name = "​<b></b>" + "Room Temperature Syringe";
        public static void Load()
        {
            ModAPI.RegisterLiquid(LukewarmSyringeSerum.ID, new LukewarmSyringeSerum());

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),

                    NameOverride = name,
                    DescriptionOverride = "Syringe that contains a liquid that keeps its container at the most desirable temperature.",

                    CategoryOverride = ModAPI.FindCategory("Polars Bear"),
                    AfterSpawn = (Instance) => { Destroy(Instance.GetComponent<SyringeBehaviour>()); Instance.GetOrAddComponent<LukewarmSyringe>(); }
                }
            );
            Mod.nailgun.SyringeNail(name, typeof(LukewarmSyringe));
        }
        public override string GetLiquidID() => LukewarmSyringeSerum.ID;
        public class LukewarmSyringeSerum : Liquid
        {
            public const string ID = "​<b></b>" + "LUKEWARM SERUM";
            public LukewarmSyringeSerum()
            {
                Color = new Color(0.26275f, 0.43922f, 0.39216f);
            }
            public override void OnEnterLimb(LimbBehaviour limb)
            {
                limb.gameObject.GetOrAddComponent<KeepLukewarmPB>();
            }
            public override void OnExitContainer(BloodContainer container)
            {
                KeepLukewarmPB kl;
                if (container.TryGetComponent(out kl) && !container.TryGetComponent<LimbBehaviour>(out _))
                {
                    Destroy(kl);
                }
            }
            public override void OnUpdate(BloodContainer container) { }
            public override void OnEnterContainer(BloodContainer container)
            {
                container.gameObject.GetOrAddComponent<KeepLukewarmPB>();
            }
            
            class KeepLukewarmPB : MonoBehaviour
            {
                PhysicalBehaviour phys;
                LimbBehaviour limb = null;
                float temp;
                void Awake()
                {
                    
                    temp = 22f; // 22°C is the default for room temperature in the game
                    phys = GetComponent<PhysicalBehaviour>();
                    if (TryGetComponent(out limb))
                    {
                        temp = 36f; // 36°C is the scientifically proven best temperature for living beings, that promotes growth
                    }
                }

                void Update()
                {
                    if (phys.Temperature != temp)
                    {
                        phys.Temperature = temp;
                    }
                    if (limb && limb.InternalTemperature != temp)
                    {
                        limb.InternalTemperature = temp;
                    }
                }
            }
        }
    }
}

//Author: Polars Bear
